///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2014 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Feb 20, 2014	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.graph;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbMessageProvider;
import com.tietoenator.trip.jxp.data.TdbSearch;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.graph.TdbGraph;
import com.tietoenator.trip.jxp.graph.TdbGraphPath;
import com.tietoenator.trip.jxp.graph.TdbGraphPathEdge;
import com.tietoenator.trip.jxp.graph.TdbGraphPathVertex;
import com.tietoenator.trip.jxp.session.TdbSession;
import java.text.DecimalFormat;

/**
 * Example of how to use obtain all paths in a graph using the graph classes in
 * TRIPjxp.
 *
 * THIS CLASS USES EXPERIMENTAL FEATURES. USE IT WITH CAUTION!
 */
public class AllPaths 
{

    private TdbSession m_session;

    public AllPaths()
            throws TdbException 
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** AllPaths TRIP graph example ****", false);
    }

    public void close() 
    {
        if (m_session != null) 
        {
            try 
            {
                m_session.logout();
            } 
            catch (TdbException ignore) 
            {
            } 
            finally 
            {
                m_session = null;
            }
        }
    }

    /**
     * Show any success message that might be forthcoming from the server
     *
     * @param prv Message provider subclass
     */
    private static void showMessages(TdbMessageProvider prv) 
    {
        for (String msg : prv.messageList()) 
        {
            System.out.println(msg);
        }
        prv.resetMessages();
    }
    
    private static void showPaths(TdbGraphPath[] paths)
    {
        TdbGraphPathVertex vtx;
        TdbGraphPathEdge edge;
        System.out.println();
        System.out.println(Integer.toString(paths.length) + " paths found.\n");
        for (TdbGraphPath gp : paths)
        {
            vtx = gp.getStartVertex();
            while (vtx != null)
            {
                System.out.print(vtx.getDatabaseName());
                System.out.print(".");
                System.out.print(vtx.getRecordId());
                
                edge = vtx.getOutputEdge();
                if (edge != null)
                {
                    System.out.print(" --(");
                    System.out.print(edge.getName());
                    System.out.print(")--> ");

                    vtx = edge.getTarget();
                }
                else
                {
                    vtx = null;
                }
            }
            System.out.println();
        }
    }

    private String readnoblank(Console stdin, String prefix, String defaultValue)
            throws TdbException 
    {
        String inval;
        do 
        {
            inval = stdin.readln(prefix, defaultValue, true);
        } 
        while (inval.length() == 0);
        return inval;

    }

    public void process(Console stdin) throws TdbException 
    {
        TdbGraph graph;
        String graphDbName;
        String sourceDbName = "";
        String targetDbName;
        String edgeName;
        int sourceRid;
        int sourceSet = 0;
        int targetRid;
        int targetSet = 0;
        String[] edgeNames = null;
        
        TdbSearch normalSearch = new TdbSearch(m_session);
        // normalSearch.execute("DEFINE FIND MIN=0");
        
        if (stdin.readbool("Use origin set",true,true))
        {
            // Database for record to explore paths from
            sourceDbName = readnoblank(stdin, "Database for origin record", "ALICE");
            normalSearch.execute("BASE " + sourceDbName);
            
            // ID of source record to explore relations from
            sourceRid = stdin.readint("Origin record ID", 1, true);
            normalSearch.execute("FIND R=" + sourceRid);
            sourceSet = normalSearch.getLastSearchSet().getSearchId();
        }
        
        if (stdin.readbool("Use destination set",true,true))
        {
            // Database for target record 
            targetDbName = readnoblank(stdin, "Database for destination record", "ALICE");
            if (!targetDbName.equalsIgnoreCase(sourceDbName))
                normalSearch.execute("BASE " + targetDbName);
            
            // ID of source record to explore relations from
            targetRid = stdin.readint("Destination record ID", 5, true);
            normalSearch.execute("FIND R=" + targetRid);
            targetSet = normalSearch.getLastSearchSet().getSearchId();
        }
        
        // The graph database to use
        graphDbName = readnoblank(stdin, "Graph database", "GRAPHBASE");
        graph = new TdbGraph(m_session, graphDbName);
        
        // The name of the relation
        edgeName = readnoblank(stdin, "Edge name (relation)", "FRIEND");
        edgeNames = new String[1];
        edgeNames[0] = edgeName;
        
        int maxDepth = stdin.readint("Max analysis depth", 10, true);
        boolean reverse = stdin.readbool("Reverse path analysis", false, true);

        TdbGraphPath[] paths = graph.getGraphSet().findPaths(sourceSet, edgeNames, 
                targetSet, maxDepth, reverse);
        
        showMessages(graph);
        showPaths(paths);
 
        normalSearch.close();
        graph.close();
    }

    public static void main(String[] args) 
    {
        AllPaths app = null;
        Console stdin = new Console();
        try 
        {
            app = new AllPaths();
            app.process(stdin);
        }
        catch (TdbException tx) 
        {
            System.err.println(tx.getDebugMessage());
        }
        catch (Exception ox) 
        {
            ox.printStackTrace(System.err);
        }
        finally 
        {
            if (app != null) 
            {
                app.close();
            }
        }
    }
}
